@@ -1,7 +1,7 @@
-import mock
+from unittest import mock
 
 from httpie.status import ExitStatus
-from utils import MockEnvironment, http, HTTP_OK
+from .utils import MockEnvironment, http, HTTP_OK
 
 
 def test_keyboard_interrupt_during_arg_parsing_exit_status(httpbin):
