@@ -5,6 +5,7 @@ from unittest import mock
 import json
 import os
 import io
+import warnings
 from urllib.request import urlopen
 
 import pytest
@@ -17,10 +18,15 @@ from httpie.cli.argtypes import (
 )
 from httpie.cli.definition import parser
 from httpie.encoding import UTF8
-from httpie.output.formatters.colors import get_lexer
+from httpie.output.formatters.colors import get_lexer, PIE_STYLE_NAMES, BUNDLED_STYLES
 from httpie.status import ExitStatus
 from .fixtures import XML_DATA_RAW, XML_DATA_FORMATTED
-from .utils import COLOR, CRLF, HTTP_OK, MockEnvironment, http, DUMMY_URL
+from .utils import COLOR, CRLF, HTTP_OK, MockEnvironment, http, DUMMY_URL, strip_colors
+
+
+# For ensuring test reproducibility, avoid using the unsorted
+# BUNDLED_STYLES set.
+SORTED_BUNDLED_STYLES = sorted(BUNDLED_STYLES)
 
 
 @pytest.mark.parametrize('stdout_isatty', [True, False])
@@ -85,6 +91,33 @@ class TestQuietFlag:
         )
         assert 'http: warning: HTTP 500' in r.stderr
 
+    @mock.patch('httpie.core.program')
+    @pytest.mark.parametrize('flags, expected_warnings', [
+        ([], 1),
+        (['-q'], 1),
+        (['-qq'], 0),
+    ])
+    # Might fail on Windows due to interference from other warnings.
+    @pytest.mark.xfail
+    def test_quiet_on_python_warnings(self, test_patch, httpbin, flags, expected_warnings):
+        def warn_and_run(*args, **kwargs):
+            warnings.warn('warning!!')
+            return ExitStatus.SUCCESS
+
+        test_patch.side_effect = warn_and_run
+        with pytest.warns(None) as record:
+            http(*flags, httpbin + '/get')
+
+        assert len(record) == expected_warnings
+
+    def test_double_quiet_on_error(self, httpbin):
+        r = http(
+            '-qq', '--check-status', '$$$this.does.not.exist$$$',
+            tolerate_error_exit_status=True,
+        )
+        assert not r
+        assert 'Couldn’t resolve the given hostname' in r.stderr
+
     @pytest.mark.parametrize('quiet_flags', QUIET_SCENARIOS)
     @mock.patch('httpie.cli.argtypes.AuthCredentials._getpass',
                 new=lambda self, prompt: 'password')
@@ -215,6 +248,43 @@ class TestColors:
         assert get_lexer('xxx/yyy') is None
 
 
+@pytest.mark.parametrize("endpoint", [
+    "/encoding/utf8",
+    "/html",
+    "/json",
+    "/xml",
+])
+def test_ensure_contents_colored(httpbin, endpoint):
+    env = MockEnvironment(colors=256)
+    r = http('--body', 'GET', httpbin + endpoint, env=env)
+    assert COLOR in r
+
+
+@pytest.mark.parametrize('style', PIE_STYLE_NAMES)
+def test_ensure_meta_is_colored(httpbin, style):
+    env = MockEnvironment(colors=256)
+    r = http('--meta', '--style', style, 'GET', httpbin + '/get', env=env)
+    assert COLOR in r
+
+
+@pytest.mark.parametrize('style', SORTED_BUNDLED_STYLES)
+@pytest.mark.parametrize('msg', [
+    '',
+    ' ',
+    ' OK',
+    ' OK ',
+    ' CUSTOM ',
+])
+def test_ensure_status_code_is_shown_on_all_themes(http_server, style, msg):
+    env = MockEnvironment(colors=256)
+    r = http('--style', style,
+             http_server + '/status/msg',
+             '--raw', msg, env=env)
+
+    # Trailing space is stripped away.
+    assert 'HTTP/1.0 200' + msg.rstrip() in strip_colors(r)
+
+
 class TestPrettyOptions:
     """Test the --pretty handling."""
 
