@@ -1,12 +1,16 @@
 import json
 import os
 import shutil
+from contextlib import contextmanager
 from datetime import datetime
 from unittest import mock
+from pathlib import Path
+from typing import Iterator
 
 import pytest
 
 from .fixtures import FILE_PATH_ARG, UNICODE
+from httpie.context import Environment
 from httpie.encoding import UTF8
 from httpie.plugins import AuthPlugin
 from httpie.plugins.builtin import HTTPBasicAuth
@@ -14,7 +18,8 @@ from httpie.plugins.registry import plugin_manager
 from httpie.sessions import Session
 from httpie.utils import get_expired_cookies
 from .test_auth_plugins import basic_auth
-from .utils import HTTP_OK, MockEnvironment, http, mk_config_dir
+from .utils import DUMMY_HOST, HTTP_OK, MockEnvironment, http, mk_config_dir
+from base64 import b64encode
 
 
 class SessionTestBase:
@@ -143,6 +148,24 @@ class TestSessionFlow(SessionTestBase):
         # Should be the same as before r3.
         assert r2.json == r4.json
 
+    def test_session_overwrite_header(self, httpbin):
+        self.start_session(httpbin)
+
+        r2 = http('--session=test', 'GET', httpbin.url + '/get',
+                  'Hello:World2', env=self.env())
+        assert HTTP_OK in r2
+        assert r2.json['headers']['Hello'] == 'World2'
+
+        r3 = http('--session=test', 'GET', httpbin.url + '/get',
+                  'Hello:World2', 'Hello:World3', env=self.env())
+        assert HTTP_OK in r3
+        assert r3.json['headers']['Hello'] == 'World2,World3'
+
+        r3 = http('--session=test', 'GET', httpbin.url + '/get',
+                  'Hello:', 'Hello:World3', env=self.env())
+        assert HTTP_OK in r3
+        assert 'Hello' not in r3.json['headers']['Hello']
+
 
 class TestSession(SessionTestBase):
     """Stand-alone session tests."""
@@ -184,9 +207,9 @@ class TestSession(SessionTestBase):
         """
         self.start_session(httpbin)
         session_data = {
-            "headers": {
-                "cookie": "...",
-                "zzz": "..."
+            'headers': {
+                'cookie': '...',
+                'zzz': '...'
             }
         }
         session_path = self.config_dir / 'session-data.json'
@@ -280,8 +303,6 @@ class TestSession(SessionTestBase):
         assert f'Authorization: {header}' in r2
         plugin_manager.unregister(Plugin)
 
-    @mock.patch('httpie.cli.argtypes.AuthCredentials._getpass',
-                new=lambda self, prompt: 'password')
     def test_auth_plugin_prompt_password_in_session(self, httpbin):
         self.start_session(httpbin)
         session_path = self.config_dir / 'test-session.json'
@@ -290,17 +311,22 @@ class TestSession(SessionTestBase):
             auth_type = 'test-prompted'
 
             def get_auth(self, username=None, password=None):
-                return basic_auth()
+                basic_auth_header = 'Basic ' + b64encode(self.raw_auth.encode()).strip().decode('latin1')
+                return basic_auth(basic_auth_header)
 
         plugin_manager.register(Plugin)
 
-        r1 = http(
-            '--session', str(session_path),
-            httpbin + '/basic-auth/user/password',
-            '--auth-type',
-            Plugin.auth_type,
-            '--auth', 'user:',
-        )
+        with mock.patch(
+            'httpie.cli.argtypes.AuthCredentials._getpass',
+            new=lambda self, prompt: 'password'
+        ):
+            r1 = http(
+                '--session', str(session_path),
+                httpbin + '/basic-auth/user/password',
+                '--auth-type',
+                Plugin.auth_type,
+                '--auth', 'user',
+            )
 
         r2 = http(
             '--session', str(session_path),
@@ -308,6 +334,13 @@ class TestSession(SessionTestBase):
         )
         assert HTTP_OK in r1
         assert HTTP_OK in r2
+
+        # additional test for issue: https://github.com/httpie/httpie/issues/1098
+        with open(session_path) as session_file:
+            session_file_lines = ''.join(session_file.readlines())
+            assert "\"type\": \"test-prompted\"" in session_file_lines
+            assert "\"raw_auth\": \"user:password\"" in session_file_lines
+
         plugin_manager.unregister(Plugin)
 
     def test_auth_type_stored_in_session_file(self, httpbin):
@@ -330,7 +363,7 @@ class TestSession(SessionTestBase):
              )
         updated_session = json.loads(self.session_path.read_text(encoding=UTF8))
         assert updated_session['auth']['type'] == 'test-saved'
-        assert updated_session['auth']['raw_auth'] == "user:password"
+        assert updated_session['auth']['raw_auth'] == 'user:password'
         plugin_manager.unregister(Plugin)
 
 
@@ -339,12 +372,12 @@ class TestExpiredCookies(CookieTestBase):
     @pytest.mark.parametrize(
         'initial_cookie, expired_cookie',
         [
-            ({'id': {'value': 123}}, 'id'),
-            ({'id': {'value': 123}}, 'token')
+            ({'id': {'value': 123}}, {'name': 'id'}),
+            ({'id': {'value': 123}}, {'name': 'token'})
         ]
     )
-    def test_removes_expired_cookies_from_session_obj(self, initial_cookie, expired_cookie, httpbin):
-        session = Session(self.config_dir)
+    def test_removes_expired_cookies_from_session_obj(self, initial_cookie, expired_cookie, httpbin, mock_env):
+        session = Session(self.config_dir, env=mock_env, session_id=None, bound_host=None)
         session['cookies'] = initial_cookie
         session.remove_cookies([expired_cookie])
         assert expired_cookie not in session.cookies
@@ -495,3 +528,309 @@ class TestCookieStorage(CookieTestBase):
         updated_session = json.loads(self.session_path.read_text(encoding=UTF8))
 
         assert updated_session['cookies']['cookie1']['value'] == expected
+
+
+@pytest.fixture
+def basic_session(httpbin, tmp_path):
+    session_path = tmp_path / 'session.json'
+    http(
+        '--session', str(session_path),
+        httpbin + '/get'
+    )
+    return session_path
+
+
+@contextmanager
+def open_session(path: Path, env: Environment, read_only: bool = False) -> Iterator[Session]:
+    session = Session(path, env, session_id='test', bound_host=DUMMY_HOST)
+    session.load()
+    yield session
+    if not read_only:
+        session.save()
+
+
+@contextmanager
+def open_raw_session(path: Path, read_only: bool = False) -> None:
+    with open(path) as stream:
+        raw_session = json.load(stream)
+
+    yield raw_session
+
+    if not read_only:
+        with open(path, 'w') as stream:
+            json.dump(raw_session, stream)
+
+
+def read_stderr(env: Environment) -> bytes:
+    env.stderr.seek(0)
+    stderr_data = env.stderr.read()
+    if isinstance(stderr_data, str):
+        return stderr_data.encode()
+    else:
+        return stderr_data
+
+
+def test_old_session_version_saved_as_is(basic_session, mock_env):
+    with open_session(basic_session, mock_env) as session:
+        session['__meta__'] = {'httpie': '0.0.1'}
+
+    with open_session(basic_session, mock_env, read_only=True) as session:
+        assert session['__meta__']['httpie'] == '0.0.1'
+
+
+def test_old_session_cookie_layout_warning(basic_session, mock_env):
+    with open_session(basic_session, mock_env) as session:
+        # Use the old layout & set a cookie
+        session['cookies'] = {}
+        session.cookies.set('foo', 'bar')
+
+    assert read_stderr(mock_env) == b''
+
+    with open_session(basic_session, mock_env, read_only=True) as session:
+        assert b'Outdated layout detected' in read_stderr(mock_env)
+
+
+@pytest.mark.parametrize('cookies, expect_warning', [
+    # Old-style cookie format
+    (
+        # Without 'domain' set
+        {'foo': {'value': 'bar'}},
+        True
+    ),
+    (
+        # With 'domain' set to empty string
+        {'foo': {'value': 'bar', 'domain': ''}},
+        True
+    ),
+    (
+        # With 'domain' set to null
+        {'foo': {'value': 'bar', 'domain': None}},
+        False,
+    ),
+    (
+        # With 'domain' set to a URL
+        {'foo': {'value': 'bar', 'domain': DUMMY_HOST}},
+        False,
+    ),
+    # New style cookie format
+    (
+        # Without 'domain' set
+        [{'name': 'foo', 'value': 'bar'}],
+        False
+    ),
+    (
+        # With 'domain' set to empty string
+        [{'name': 'foo', 'value': 'bar', 'domain': ''}],
+        False
+    ),
+    (
+        # With 'domain' set to null
+        [{'name': 'foo', 'value': 'bar', 'domain': None}],
+        False,
+    ),
+    (
+        # With 'domain' set to a URL
+        [{'name': 'foo', 'value': 'bar', 'domain': DUMMY_HOST}],
+        False,
+    ),
+])
+def test_cookie_security_warnings_on_raw_cookies(basic_session, mock_env, cookies, expect_warning):
+    with open_raw_session(basic_session) as raw_session:
+        raw_session['cookies'] = cookies
+
+    with open_session(basic_session, mock_env, read_only=True):
+        warning = b'Outdated layout detected'
+        stderr = read_stderr(mock_env)
+
+        if expect_warning:
+            assert warning in stderr
+        else:
+            assert warning not in stderr
+
+
+def test_old_session_cookie_layout_loading(basic_session, httpbin, mock_env):
+    with open_session(basic_session, mock_env) as session:
+        # Use the old layout & set a cookie
+        session['cookies'] = {}
+        session.cookies.set('foo', 'bar')
+
+    response = http(
+        '--session', str(basic_session),
+        httpbin + '/cookies'
+    )
+    assert response.json['cookies'] == {'foo': 'bar'}
+
+
+@pytest.mark.parametrize('layout_type', [
+    dict, list
+])
+def test_session_cookie_layout_preservation(basic_session, mock_env, layout_type):
+    with open_session(basic_session, mock_env) as session:
+        session['cookies'] = layout_type()
+        session.cookies.set('foo', 'bar')
+        session.save()
+
+    with open_session(basic_session, mock_env, read_only=True) as session:
+        assert isinstance(session['cookies'], layout_type)
+
+
+@pytest.mark.parametrize('layout_type', [
+    dict, list
+])
+def test_session_cookie_layout_preservation_on_new_cookies(basic_session, httpbin, mock_env, layout_type):
+    with open_session(basic_session, mock_env) as session:
+        session['cookies'] = layout_type()
+        session.cookies.set('foo', 'bar')
+        session.save()
+
+    http(
+        '--session', str(basic_session),
+        httpbin + '/cookies/set/baz/quux'
+    )
+
+    with open_session(basic_session, mock_env, read_only=True) as session:
+        assert isinstance(session['cookies'], layout_type)
+
+
+@pytest.mark.parametrize('headers, expect_warning', [
+    # Old-style header format
+    (
+        {},
+        False
+    ),
+    (
+        {'Foo': 'bar'},
+        True
+    ),
+    # New style header format
+    (
+        [],
+        False
+    ),
+    (
+        [{'name': 'Foo', 'value': 'Bar'}],
+        False
+    ),
+])
+def test_headers_old_layout_warning(basic_session, mock_env, headers, expect_warning):
+    with open_raw_session(basic_session) as raw_session:
+        raw_session['headers'] = headers
+
+    with open_session(basic_session, mock_env, read_only=True):
+        warning = b'Outdated layout detected'
+        stderr = read_stderr(mock_env)
+
+        if expect_warning:
+            assert warning in stderr
+        else:
+            assert warning not in stderr
+
+
+def test_outdated_layout_mixed(basic_session, mock_env):
+    with open_raw_session(basic_session) as raw_session:
+        raw_session['headers'] = {'Foo': 'Bar'}
+        raw_session['cookies'] = {
+            'cookie': {
+                'value': 'value'
+            }
+        }
+
+    with open_session(basic_session, mock_env, read_only=True):
+        stderr = read_stderr(mock_env)
+        # We should only see 1 warning.
+        assert stderr.count(b'Outdated layout') == 1
+
+
+def test_old_session_header_layout_loading(basic_session, httpbin, mock_env):
+    with open_session(basic_session, mock_env) as session:
+        # Use the old layout & set a header
+        session['headers'] = {}
+        session._headers.add('Foo', 'Bar')
+
+    response = http(
+        '--session', str(basic_session),
+        httpbin + '/get'
+    )
+    assert response.json['headers']['Foo'] == 'Bar'
+
+
+@pytest.mark.parametrize('layout_type', [
+    dict, list
+])
+def test_session_header_layout_preservation(basic_session, mock_env, layout_type):
+    with open_session(basic_session, mock_env) as session:
+        session['headers'] = layout_type()
+        session._headers.add('Foo', 'Bar')
+
+    with open_session(basic_session, mock_env, read_only=True) as session:
+        assert isinstance(session['headers'], layout_type)
+
+
+@pytest.mark.parametrize('layout_type', [
+    dict, list
+])
+def test_session_header_layout_preservation_on_new_headers(basic_session, httpbin, mock_env, layout_type):
+    with open_session(basic_session, mock_env) as session:
+        session['headers'] = layout_type()
+        session._headers.add('Foo', 'Bar')
+
+    http(
+        '--session', str(basic_session),
+        httpbin + '/get',
+        'Baz:Quux'
+    )
+
+    with open_session(basic_session, mock_env, read_only=True) as session:
+        assert isinstance(session['headers'], layout_type)
+
+
+def test_session_multiple_headers_with_same_name(basic_session, httpbin):
+    http(
+        '--session', str(basic_session),
+        httpbin + '/get',
+        'Foo:bar',
+        'Foo:baz',
+        'Foo:bar'
+    )
+
+    r = http(
+        '--offline',
+        '--session', str(basic_session),
+        httpbin + '/get',
+    )
+    assert r.count('Foo: bar') == 2
+    assert 'Foo: baz' in r
+
+
+@pytest.mark.parametrize(
+    'server, expected_cookies',
+    [
+        (
+            pytest.lazy_fixture('localhost_http_server'),
+            {'secure_cookie': 'foo', 'insecure_cookie': 'bar'}
+        ),
+        (
+            pytest.lazy_fixture('remote_httpbin'),
+            {'insecure_cookie': 'bar'}
+        )
+    ]
+)
+def test_secure_cookies_on_localhost(mock_env, tmp_path, server, expected_cookies):
+    session_path = tmp_path / 'session.json'
+    http(
+        '--session', str(session_path),
+        server + '/cookies/set',
+        'secure_cookie==foo',
+        'insecure_cookie==bar'
+    )
+
+    with open_session(session_path, mock_env) as session:
+        for cookie in session.cookies:
+            if cookie.name == 'secure_cookie':
+                cookie.secure = True
+
+    r = http(
+        '--session', str(session_path),
+        server + '/cookies'
+    )
+    assert r.json == {'cookies': expected_cookies}
