@@ -1,9 +1,16 @@
 import os
+import json
+import sys
+import subprocess
+import time
+import contextlib
+import httpie.__main__ as main
 
 import pytest
 
 from httpie.cli.exceptions import ParseError
 from httpie.client import FORM_CONTENT_TYPE
+from httpie.compat import is_windows
 from httpie.status import ExitStatus
 from .utils import (
     MockEnvironment, StdinBytesIO, http,
@@ -11,6 +18,8 @@ from .utils import (
 )
 from .fixtures import FILE_PATH_ARG, FILE_PATH, FILE_CONTENT
 
+MAX_RESPONSE_WAIT_TIME = 5
+
 
 def test_chunked_json(httpbin_with_chunked_support):
     r = http(
@@ -70,6 +79,114 @@ def test_chunked_stdin_multiple_chunks(httpbin_with_chunked_support):
     assert r.count(FILE_CONTENT) == 4
 
 
+def test_chunked_raw(httpbin_with_chunked_support):
+    r = http(
+        '--verbose',
+        '--chunked',
+        httpbin_with_chunked_support + '/post',
+        '--raw',
+        json.dumps({'a': 1, 'b': '2fafds', 'c': '🥰'}),
+    )
+    assert HTTP_OK in r
+    assert 'Transfer-Encoding: chunked' in r
+
+
+@contextlib.contextmanager
+def stdin_processes(httpbin, *args, warn_threshold=0.1):
+    process_1 = subprocess.Popen(
+        [
+            "cat"
+        ],
+        stdin=subprocess.PIPE,
+        stdout=subprocess.PIPE
+    )
+    process_2 = subprocess.Popen(
+        [
+            sys.executable,
+            main.__file__,
+            "POST",
+            httpbin + "/post",
+            *args
+        ],
+        stdin=process_1.stdout,
+        stderr=subprocess.PIPE,
+        env={
+            **os.environ,
+            "HTTPIE_STDIN_READ_WARN_THRESHOLD": str(warn_threshold)
+        }
+    )
+    try:
+        yield process_1, process_2
+    finally:
+        process_1.terminate()
+        process_2.terminate()
+
+
+@pytest.mark.parametrize("wait", (True, False))
+@pytest.mark.requires_external_processes
+@pytest.mark.skipif(is_windows, reason="Windows doesn't support select() calls into files")
+def test_reading_from_stdin(httpbin, wait):
+    with stdin_processes(httpbin) as (process_1, process_2):
+        process_1.communicate(timeout=0.1, input=b"bleh")
+        # Since there is data, it doesn't matter if there
+        # you wait or not.
+        if wait:
+            time.sleep(1)
+
+        try:
+            _, errs = process_2.communicate(timeout=MAX_RESPONSE_WAIT_TIME)
+        except subprocess.TimeoutExpired:
+            errs = b''
+
+        assert b'> warning: no stdin data read in 0.1s' not in errs
+
+
+@pytest.mark.requires_external_processes
+@pytest.mark.skipif(is_windows, reason="Windows doesn't support select() calls into files")
+def test_stdin_read_warning(httpbin):
+    with stdin_processes(httpbin) as (process_1, process_2):
+        # Wait before sending any data
+        time.sleep(1)
+        process_1.communicate(timeout=0.1, input=b"bleh\n")
+
+        try:
+            _, errs = process_2.communicate(timeout=MAX_RESPONSE_WAIT_TIME)
+        except subprocess.TimeoutExpired:
+            errs = b''
+
+        assert b'> warning: no stdin data read in 0.1s' in errs
+
+
+@pytest.mark.requires_external_processes
+@pytest.mark.skipif(is_windows, reason="Windows doesn't support select() calls into files")
+def test_stdin_read_warning_with_quiet(httpbin):
+    with stdin_processes(httpbin, "-qq") as (process_1, process_2):
+        # Wait before sending any data
+        time.sleep(1)
+        process_1.communicate(timeout=0.1, input=b"bleh\n")
+
+        try:
+            _, errs = process_2.communicate(timeout=MAX_RESPONSE_WAIT_TIME)
+        except subprocess.TimeoutExpired:
+            errs = b''
+
+        assert b'> warning: no stdin data read in 0.1s' not in errs
+
+
+@pytest.mark.requires_external_processes
+@pytest.mark.skipif(is_windows, reason="Windows doesn't support select() calls into files")
+def test_stdin_read_warning_blocking_exit(httpbin):
+    # Use a very large number.
+    with stdin_processes(httpbin, warn_threshold=999) as (process_1, process_2):
+        # Wait before sending any data
+        time.sleep(1)
+        process_1.communicate(timeout=0.1, input=b"some input\n")
+
+        # If anything goes wrong, and the thread starts the block this
+        # will timeout and let us know.
+        process_2.communicate(timeout=MAX_RESPONSE_WAIT_TIME)
+
+
 class TestMultipartFormDataFileUpload:
 
     def test_non_existent_file_raises_parse_error(self, httpbin):
